@@ -20,13 +20,17 @@ class CompBuilder(CompExporter):
     __data = {}
     # thermodb name (optional)
     __thermodb_name: str | None = None
+    # message
+    __message: str | None = None
     # thermodb version
     build_version = __version__
 
-    def __init__(self, thermodb_name: Optional[str] = None):
+    def __init__(self,
+                 thermodb_name: Optional[str] = None,
+                 message: Optional[str] = None):
         '''
         Initialize CompBuilder object
-        
+
         Parameters
         ----------
         thermodb_name : str
@@ -36,12 +40,17 @@ class CompBuilder(CompExporter):
         super().__init__()
         # set name
         self.__thermodb_name = thermodb_name
-        
+        # set message
+        self.__message = message if message is not None else 'CompBuilder instance created!'
+
+        # ! reset data
+        self.__data = {}
+
     @property
     def thermodb_name(self) -> str | None:
         '''
         Get thermodb name
-        
+
         Returns
         -------
         str
@@ -52,7 +61,23 @@ class CompBuilder(CompExporter):
             # return default name
             return 'thermodb'
         return self.__thermodb_name
-        
+
+    @property
+    def message(self) -> str:
+        '''
+        Get message
+
+        Returns
+        -------
+        str
+            message
+        '''
+        # check message
+        if self.__message is None:
+            # return default message
+            return 'CompBuilder instance created!'
+        return self.__message
+
     def add_data(self, name: str, value: Union[TableData, TableEquation, dict, TableMatrixData, TableMatrixEquation]):
         '''
         Add TableData/TableEquation
@@ -281,7 +306,7 @@ class CompBuilder(CompExporter):
         Returns
         -------
         res : dict
-            list of all properties and functions registered 
+            list of all properties and functions registered
         '''
         try:
             # res
@@ -307,7 +332,9 @@ class CompBuilder(CompExporter):
         except Exception as e:
             raise Exception('Checking properties failed!, ', e)
 
-    def check_property(self, thermo_name: str) -> TableData | TableMatrixData:
+    def check_property(self,
+                       thermo_name: str
+                       ) -> TableData | TableMatrixData:
         '''
         Check properties
 
@@ -326,16 +353,18 @@ class CompBuilder(CompExporter):
             return self.properties[thermo_name]
         except Exception as e:
             raise Exception('Checking properties failed!, ', e)
-        
-    def select_property(self, thermo_name: str) -> TableData | TableMatrixData:
+
+    def select_property(self,
+                        thermo_name: str
+                        ) -> TableData | TableMatrixData:
         '''
         Select a thermodynamic property
-        
+
         Parameters
         ----------
         thermo_name : str
             name of the thermodynamic property
-            
+
         Returns
         -------
         TableMatrixData | TableData
@@ -362,7 +391,9 @@ class CompBuilder(CompExporter):
         except Exception as e:
             raise Exception('Checking functions failed!, ', e)
 
-    def check_function(self, name: str) -> TableEquation | TableMatrixEquation:
+    def check_function(self,
+                       name: str
+                       ) -> TableEquation | TableMatrixEquation:
         '''
         Check functions
 
@@ -381,16 +412,18 @@ class CompBuilder(CompExporter):
             return self.functions[name]
         except Exception as e:
             raise Exception('Checking functions failed!, ', e)
-        
-    def select_function(self, function_name: str) -> TableEquation | TableMatrixEquation:
+
+    def select_function(self,
+                        function_name: str
+                        ) -> TableEquation | TableMatrixEquation:
         '''
         Select a thermodynamic function
-        
+
         Parameters
         ----------
         function_name : str
             name of the thermodynamic function
-            
+
         Returns
         -------
         TableMatrixEquation | TableEquation
@@ -401,8 +434,15 @@ class CompBuilder(CompExporter):
             return self.functions[function_name]
         except Exception as e:
             raise Exception('Selecting a function failed!, ', e)
-        
-    def select(self, thermo_name: str) -> TableData | TableMatrixData | TableEquation | TableMatrixEquation:
+
+    def select(self,
+               thermo_name: str
+               ) -> Union[
+                   TableData,
+                   TableMatrixData,
+                   TableEquation,
+                   TableMatrixEquation
+    ]:
         '''
         Select a thermodynamic property used for both functions and properties registered in the thermodb
 
@@ -414,16 +454,18 @@ class CompBuilder(CompExporter):
         Returns
         -------
         TableMatrixData | TableData | TableEquation | TableMatrixEquation
-            property defined in the thermodb 
+            property defined in the thermodb
         '''
         try:
             # SECTION 1: check if the property exists in both functions and properties
-            check_list = list(self.functions.keys()) + list(self.properties.keys())
+            check_list = list(self.functions.keys()) + \
+                list(self.properties.keys())
             # check if the property exists in both functions and properties
             if (thermo_name not in check_list):
                 # raise
-                raise Exception('Property exists in both functions and properties!')
-            
+                raise Exception(
+                    'Property exists in both functions and properties!')
+
             # SECTION 2: check if the property is a function or a property
             if thermo_name in self.functions:
                 # check if the property is a function
@@ -437,10 +479,15 @@ class CompBuilder(CompExporter):
         except Exception as e:
             raise Exception('Selecting a thermodynamic property failed!, ', e)
 
-    def retrieve(self, property_source: str, message: Optional[str] = None, symbol_format: Literal['alphabetic', 'numeric'] = 'alphabetic'):
+    def retrieve(self,
+                 property_source: str,
+                 message: Optional[str] = None,
+                 symbol_format: Literal[
+                     'alphabetic', 'numeric'
+                 ] = 'alphabetic'):
         '''
         Retrieve a thermodynamic property from the thermodb for only TableData and TableMatrixData
-        
+
         Parameters
         ----------
         property_source : str
@@ -449,17 +496,17 @@ class CompBuilder(CompExporter):
             message to display (default is None)
         symbol_format : str
             symbol format to use (default is 'alphabetic'), needed for `TableMatrixData`
-            
+
         Returns
         -------
         prop: DataResult
             property object
-            
+
         Notes
         -----
         The property source is a string containing the name of the property source and the name of the property separated by a pipe (|) character.
-        For example, 
-        
+        For example,
+
         1- 'general-data | dH_IG' means that the property is in the general-data source and the name of the property is dH_IG.
         2- 'nrtl-data | alpha_ij | methanol | ethanol' means that the property is in the nrtl-data source and the name of the property is alpha_ij and the components are methanol and ethanol.
         '''
@@ -468,7 +515,7 @@ class CompBuilder(CompExporter):
             source = property_source.split('|')
             # num
             source_num = len(source)
-            
+
             # SECTION: check message
             message = message if message is not None else f'Retrieving used for {property_source}!'
 
@@ -480,9 +527,11 @@ class CompBuilder(CompExporter):
             if isinstance(prop_src, TableData):
                 # check length
                 if source_num != 2:
-                    raise ValueError(f"Invalid source format! {property_source}")
+                    raise ValueError(
+                        f"Invalid source format! {property_source}")
                 # get property
-                prop = prop_src.get_property(source[1].strip(), message=message)
+                prop = prop_src.get_property(
+                    source[1].strip(), message=message)
                 # return
                 return prop
             elif isinstance(prop_src, TableMatrixData):
@@ -490,39 +539,46 @@ class CompBuilder(CompExporter):
                 if source_num == 2:
                     # property name full format
                     prop_name = source[1].strip()
-                    
+
                     # check if the property name is in the format of 'Alpha_i_j'
                     extracted = prop_name.split('_')
                     # count
                     if len(extracted) != 3:
-                        raise ValueError(f"Invalid source format! {property_source}, property name is required!")
-                    
+                        raise ValueError(
+                            f"Invalid source format! {property_source}, property name is required!")
+
                     # NOTE: get property (ij method)
-                    prop = prop_src.ij(prop_name, symbol_format=symbol_format, message=message)
+                    prop = prop_src.ij(
+                        prop_name, symbol_format=symbol_format, message=message)
                     # return
                     return prop
                 elif source_num == 4:
                     # get components
                     component_names = source[2:]
                     # trim
-                    component_names = [name.strip() for name in component_names]
+                    component_names = [name.strip()
+                                       for name in component_names]
                     # check length
                     if len(component_names) != 2:
-                        raise ValueError(f"Invalid source format! {property_source}, components are required!")
-                    
+                        raise ValueError(
+                            f"Invalid source format! {property_source}, components are required!")
+
                     # NOTE: get property (get_matrix_property method)
-                    prop = prop_src.get_matrix_property(source[1].strip(), component_names=component_names, 
+                    prop = prop_src.get_matrix_property(source[1].strip(), component_names=component_names,
                                                         symbol_format=symbol_format, message=message)
                     # return
                     return prop
             else:
                 raise Exception(
                     f"Property source is not a TableData object! {prop_src}")
-                
+
         except Exception as e:
             raise Exception("Retrieving failed!, ", e)
-        
-    def save(self, filename: str, file_path: Optional[str] = None) -> bool:
+
+    def save(self,
+             filename: str,
+             file_path: Optional[str] = None
+             ) -> bool:
         """
         Saves the instance to a file using pickle
 
