@@ -5,8 +5,16 @@
 import os
 import yaml
 import pandas as pd
-from typing import TypedDict, List, Optional, Literal, Tuple
+import logging
+from typing import (
+    List,
+    Optional,
+    Literal,
+    Dict,
+    Any
+)
 import json
+import re
 # local
 from ..data import TableTypes
 from ..models import DataBookTableTypes
@@ -136,6 +144,125 @@ class ManageData():
         # print(type(reference))
         return reference
 
+    @staticmethod
+    def load_custom_reference(
+        custom_ref: CustomRef,
+        save_to_file: bool = False,
+        file_format: Literal['txt', 'yml'] = 'txt',
+        file_name: Optional[str] = None,
+        output_dir: Optional[str] = None
+    ) -> Dict[str, Any] | None:
+        '''
+        Load custom reference data from file
+
+        Parameters
+        ----------
+        custom_ref : CustomRef | None
+            custom reference object
+        save_to_file : bool, optional
+            If True, save the custom reference to a txt file, by default False
+        file_format : Literal['txt', 'yml'], optional
+            Format of the file to save the custom reference, by default 'txt'. Options are 'txt' or 'yml'.
+        file_name : str, optional
+            Name of the file to save the custom reference, by default None.
+        output_dir : str, optional
+            Directory to save the custom reference file, by default None.
+
+        Returns
+        -------
+        dict[str, dict]
+            custom reference data
+        '''
+        try:
+            # NOTE: check custom_ref
+            if not isinstance(custom_ref, CustomRef):
+                logging.warning("Custom reference is not provided or invalid!")
+                return {}
+
+            # NOTE: load custom reference
+            if custom_ref:
+                # check if directory is provided
+                if output_dir is None:
+                    # use current working directory
+                    output_dir = os.getcwd()
+
+                # check if directory exists
+                if not os.path.exists(output_dir):
+                    # create directory
+                    os.makedirs(output_dir)
+
+                # SECTION: load custom reference
+                ref = custom_ref.load_ref()
+
+                # check if ref is empty
+                if not ref:
+                    logging.warning("Custom reference is empty!")
+                    return {}
+
+                # check if file_name is provided
+                if file_name is None:
+                    # use default file name
+                    if file_format == 'txt':
+                        file_name = "custom_reference.txt"
+                    elif file_format == 'yml':
+                        file_name = "custom_reference.yml"
+                    else:
+                        logging.warning(
+                            "Invalid file format! Use 'txt' or 'yml'.")
+                        return {}
+                else:
+                    # set name
+                    file_name = file_name.strip() + f".{file_format}"
+
+                if save_to_file:
+                    # build the full file path using directory and file_name
+                    file_path = os.path.join(output_dir, file_name)
+
+                    # check file format
+                    if file_format == 'txt':
+                        # ! to txt file
+                        with open(file_path, "w") as f:
+                            # `indent=4` for nice formatting
+                            json.dump(ref, f, indent=4)
+                    elif file_format == 'yml':
+                        # ! to yaml file
+                        with open(file_path, "w") as f:
+                            # `default_flow_style=False` for nice formatting
+                            yaml.dump(ref, f, sort_keys=False)
+                    else:
+                        logging.warning(
+                            "Invalid file format! Use 'txt' or 'yml'.")
+
+                return ref
+            else:
+                return {}
+        except Exception as e:
+            logging.error(f"Error loading custom reference: {e}")
+            return {}
+
+    def select_reference(self, reference: str) -> dict:
+        '''
+        Select a reference from the loaded references
+
+        Parameters
+        ----------
+        reference : str
+            reference name
+
+        '''
+        try:
+            # check
+            if reference not in self.__reference['REFERENCES']:
+                raise ValueError(f"Reference {reference} not found!")
+
+            # get reference
+            selected_ref = self.__reference['REFERENCES'][reference]
+
+            # return
+            return selected_ref
+        except Exception as e:
+            raise Exception(f"select reference error! {e}")
+
     def load_symbols(self, custom_ref: CustomRef | None) -> dict:
         '''
         Load symbols used in the databooks
@@ -191,25 +318,27 @@ class ManageData():
             descriptions = {}
 
             for key, value in references.items():
+                # databook name
+                databook_name = key
                 # databook id
                 DATABOOK_ID = value.get('DATABOOK-ID', None)
 
                 # init
-                descriptions[key] = {}
+                descriptions[databook_name] = {}
                 # set
-                descriptions[key]['DATABOOK-ID'] = DATABOOK_ID
+                descriptions[databook_name]['DATABOOK-ID'] = DATABOOK_ID
 
                 # check tables
                 for table, table_data in value.get('TABLES', {}).items():
                     # check
                     if 'DESCRIPTION' in table_data:
-                        descriptions[key][table] = {
+                        descriptions[databook_name][table] = {
                             'DATABOOK-ID': DATABOOK_ID,
                             'TABLE-ID': table_data.get('TABLE-ID', None),
                             'DESCRIPTION': table_data['DESCRIPTION']
                         }
                     else:
-                        descriptions[key][table] = {
+                        descriptions[databook_name][table] = {
                             'DATABOOK-ID': DATABOOK_ID,
                             'TABLE-ID': table_data.get('TABLE-ID', None),
                             'DESCRIPTION': None
@@ -268,6 +397,33 @@ class ManageData():
         except Exception as e:
             raise Exception(f"description loading error! {e}")
 
+    def get_descriptions_by_databook(self, databook: str) -> dict:
+        '''
+        Get descriptions by databook
+
+        Parameters
+        ----------
+        databook : str
+            databook name
+
+        Returns
+        -------
+        descriptions : dict
+            descriptions of the databook
+        '''
+        try:
+            # check
+            if databook not in self.__description:
+                raise ValueError(f"Databook {databook} not found!")
+
+            # get descriptions
+            descriptions = self.__description[databook]
+
+            # return
+            return descriptions
+        except Exception as e:
+            raise Exception(f"description loading error! {e}")
+
     def get_databook_bulk(self) -> dict[str, list[DataBookTableTypes]]:
         '''
         Get databook bulk
@@ -301,13 +457,27 @@ class ManageData():
                     # <class 'str' > <class 'dict' >
                     # print(type(table), type(table_data))
 
-                    # description
+                    # NOTE: description
                     description = table_data.get('DESCRIPTION', None)
 
-                    # table id
+                    # NOTE: table id
                     table_id = table_data.get('TABLE-ID', -1)
 
-                    # * check
+                    # NOTE: table values
+                    table_values = table_data.get('VALUES', None)
+
+                    # NOTE: table structure
+                    table_structure = table_data.get('STRUCTURE', None)
+
+                    # NOTE: table reference
+                    external_references = table_data.get(
+                        'EXTERNAL-REFERENCES', None)
+
+                    # SECTION: reference indicators
+                    # ? DATA,
+                    # ? EQUATIONS,
+                    # ? MATRIX-DATA OR MATRIX-SYMBOL,
+                    # ? MATRIX-EQUATIONS
                     # ! check EQUATIONS exists
                     if 'EQUATIONS' in table_data:
                         # eq
@@ -316,15 +486,22 @@ class ManageData():
                             # save
                             _eq.append(eq_data)
 
+                        # NOTE: parse equations
+                        _eq_formatted = self.__eq_formatter(_eq)
+
                         # save
                         tables.append({
                             'table_id': table_id,
                             'table': table,
                             'description': description,
-                            'equations': _eq,
+                            'equations': _eq_formatted,
                             'data': None,
                             'matrix_equations': None,
                             'matrix_data': None,
+                            'table_type': TableTypes.EQUATIONS.value,
+                            'table_values': table_values,
+                            'table_structure': table_structure,
+                            'external_references': external_references
                         })
                         # reset
                         _eq = []
@@ -335,14 +512,6 @@ class ManageData():
                         for eq, eq_data in table_data['MATRIX-EQUATIONS'].items():
                             # save
                             _eq.append(eq_data)
-                            
-                        # matrix-symbol
-                        # matrix_symbol = table_data.get('MATRIX-SYMBOL', None)
-                        # # embedded symbol
-                        # if matrix_symbol:
-                        #     _eq.append({
-                        #         'MATRIX-SYMBOL': matrix_symbol
-                        #     })
 
                         # save
                         tables.append({
@@ -353,13 +522,26 @@ class ManageData():
                             'data': None,
                             'matrix_equations': _eq,
                             'matrix_data': None,
+                            'table_type': TableTypes.MATRIX_EQUATIONS.value,
+                            'table_values': table_values,
+                            'table_structure': table_structure,
+                            'external_references': external_references
                         })
                         # reset
                         _eq = []
                     # ! check DATA
                     elif 'DATA' in table_data:
                         # data
-                        data = table_data['DATA']
+                        data = table_data.get('DATA', [])
+
+                        # REVIEW: generate table structure
+                        table_structure = data if table_structure is None else table_structure
+                        # set
+                        if isinstance(data, list):
+                            # check data
+                            if len(data) == 0:
+                                data = table_structure
+
                         # save
                         tables.append({
                             'table_id': table_id,
@@ -369,17 +551,39 @@ class ManageData():
                             'data': data,
                             'matrix_equations': None,
                             'matrix_data': None,
+                            'table_type': TableTypes.DATA.value,
+                            'table_values': table_values,
+                            'table_structure': table_structure,
+                            'external_references': external_references
                         })
                     # ! check MATRIX-DATA
-                    elif 'MATRIX-DATA' in table_data:
-                        # matrix-data
-                        matrix_data = table_data['MATRIX-DATA']
-                        # matrix-symbol
+                    elif ('MATRIX-DATA' in table_data) or ('MATRIX-SYMBOL' in table_data):
+                        # matrix-data (data)
+                        # NOTE: matrix-data
+                        # including COLUMNS, SYMBOL, UNIT, CONVERSION, MATRIX-SYMBOL
+                        matrix_data = table_data.get('MATRIX-DATA', {})
+
+                        # NOTE: matrix-symbol
                         matrix_symbol = table_data.get('MATRIX-SYMBOL', None)
+
                         # embedded symbol
                         if matrix_symbol:
                             matrix_data['MATRIX-SYMBOL'] = matrix_symbol
-                        
+
+                        # NOTE: table items
+                        table_items = table_data.get('ITEMS', None)
+
+                        # embedded symbol
+                        if table_items:
+                            matrix_data['ITEMS'] = table_items
+
+                        # NOTE: table structure
+                        if table_structure is not None:
+                            # update
+                            for k, v in table_structure.items():
+                                # update
+                                matrix_data[k] = v
+
                         # save
                         tables.append({
                             'table_id': table_id,
@@ -388,13 +592,17 @@ class ManageData():
                             'equations': None,
                             'data': None,
                             'matrix_equations': None,
-                            'matrix_data': matrix_data
+                            'matrix_data': matrix_data,
+                            'table_type': TableTypes.MATRIX_DATA.value,
+                            'table_values': table_values,
+                            'table_structure': table_structure,
+                            'table_items': table_items,
+                            'external_references': external_references
                         })
 
                 # save
                 databook_list[databook] = tables
-                # log
-                # print(type(databook_list))
+
             # return
             return databook_list
         except Exception as e:
@@ -443,7 +651,13 @@ class ManageData():
         except Exception as e:
             raise Exception(f"databook loading error! {e}")
 
-    def get_databook_id(self, databook: str, res_format: Literal['str', 'json', 'dict', 'int'] = 'json') -> str | int | dict[str, str]:
+    def get_databook_id(
+        self,
+        databook: str,
+        res_format: Literal[
+            'str', 'json', 'dict', 'int'
+        ] = 'json'
+    ) -> str | int | dict[str, str]:
         '''
         Get databook id
 
@@ -489,7 +703,7 @@ class ManageData():
         except Exception as e:
             raise Exception(f"databook id loading error! {e}")
 
-    def get_tables(self, databook) -> tuple[list[list[str]], pd.DataFrame, str]:
+    def get_tables(self, databook) -> tuple[list[list[str]], pd.DataFrame, str, dict[str, str]]:
         '''
         Get a table list of selected databook
 
@@ -546,7 +760,7 @@ class ManageData():
             tables_df = pd.DataFrame(
                 tables, columns=[column_name, "Type", "Id"])
             # return
-            return tables, tables_df, tables_json
+            return tables, tables_df, tables_json, tables_dict
         except Exception as e:
             raise Exception(f"table loading err! {e}")
 
@@ -592,9 +806,11 @@ class ManageData():
         except Exception as e:
             raise Exception(f"table info loading err! {e}")
 
-    def get_table(self, databook: str | int | list[DataBookTableTypes], table: int | str) -> DataBookTableTypes:
+    def get_table(self,
+                  databook: str | int | list[DataBookTableTypes],
+                  table: int | str) -> DataBookTableTypes:
         '''
-        Get a table list of selected databook
+        Get a table with respect to databook and table id or name
 
         Parameters
         ----------
@@ -605,8 +821,8 @@ class ManageData():
 
         Returns
         -------
-        tables : list
-            table list of selected databook
+        table : DataBookTableTypes
+            selected table based on databook and table id or name
         '''
         try:
             # select databook
@@ -620,15 +836,22 @@ class ManageData():
 
             # ! if databook list
             # list
-            selected_tb: DataBookTableTypes = {
-                'table_id': 'Table not found!',
-                'table': 'Table not found!',
-                'description': None,
-                "data": None,
-                "equations": None,
-                "matrix_data": None,
-                "matrix_equations": None
-            }
+            # selected_tb: DataBookTableTypes
+            selected_tb = None
+
+            # SECTION: set table type
+            def set_table_type(tb):
+                # check table type
+                if 'equations' in tb and tb['equations'] is not None:
+                    return TableTypes.EQUATIONS.value
+                elif 'data' in tb and tb['data'] is not None:
+                    return TableTypes.DATA.value
+                elif 'matrix_data' in tb and tb['matrix_data'] is not None:
+                    return TableTypes.MATRIX_DATA.value
+                elif 'matrix_equations' in tb and tb['matrix_equations'] is not None:
+                    return TableTypes.MATRIX_EQUATIONS.value
+                else:
+                    raise Exception("table type unknown!")
 
             # by table id
             if isinstance(table, int):
@@ -636,7 +859,11 @@ class ManageData():
                 for i, tb in enumerate(databook_set):
                     # check table id
                     if i == table:
-                        selected_tb = tb
+                        selected_tb = DataBookTableTypes(**tb)
+                        # NOTE: check table type
+                        # tb_name_ = selected_tb['table']
+                        # set
+                        selected_tb['table_type'] = set_table_type(tb)
                         break
             # by table name
             elif isinstance(table, str):
@@ -644,17 +871,29 @@ class ManageData():
                 for i, tb in enumerate(databook_set):
                     # check table name
                     if tb['table'] == table:
-                        selected_tb = tb
+                        selected_tb = DataBookTableTypes(**tb)
+                        # NOTE: check table type
+                        # tb_name_ = selected_tb['table']
+                        # set
+                        selected_tb['table_type'] = set_table_type(tb)
                         break
             else:
                 raise Exception("Invalid table type")
 
+            # NOTE: check selected table
+            if selected_tb is None:
+                raise Exception("table not found!")
             # return
             return selected_tb
         except Exception as e:
             raise Exception(f"table loading err! {e}")
 
-    def get_table_id(self, databook: str | int, table: str, res_format: Literal['str', 'json', 'dict'] = 'json') -> str | dict[str, str]:
+    def get_table_id(self,
+                     databook: str | int,
+                     table: str,
+                     res_format: Literal[
+                         'str', 'json', 'dict'
+                     ] = 'json') -> str | dict[str, str]:
         '''
         Get table id
 
@@ -702,7 +941,40 @@ class ManageData():
         except Exception as e:
             raise Exception(f"table id loading err! {e}")
 
-    def find_databook(self, databook: str | int) -> tuple[list[DataBookTableTypes], str, int]:
+    def get_table_values(self, databook: str | int, table: str | int):
+        '''
+        Get table values
+
+        Parameters
+        ----------
+        databook : str | int
+            databook name or id
+        table : str | int
+            table name or id
+
+        Returns
+        -------
+        table_values : list
+            table values
+        '''
+        try:
+            # NOTE: find databook
+            db, db_name, db_id = self.find_databook(databook)
+
+            # NOTE: find table
+            tb_id, tb_name = self.find_table(db_name, table)
+
+            # NOTE: get table
+            tb = self.get_table(db_name, tb_name)
+
+            return tb
+
+        except Exception as e:
+            raise Exception(f"table values loading err! {e}")
+
+    def find_databook(self,
+                      databook: str | int
+                      ) -> tuple[list[DataBookTableTypes], str, int]:
         '''
         Find a databook
 
@@ -902,3 +1174,234 @@ class ManageData():
             return {}
         except Exception as e:
             raise Exception('Finding table source failed!', e)
+
+    def __parse_equation(self, eq_data: List[str]):
+        '''
+        Parse equation data into a dictionary including equation args, parameters, returns
+
+        Parameters
+        ----------
+        eq_data : dict
+            equation data
+        '''
+        try:
+            # check
+            if not isinstance(eq_data, list):
+                raise ValueError("eq_data must be a list!")
+
+            # init
+            eq_dict = {}
+            eq_dict['ARGS'] = {}
+            eq_dict['PARMS'] = {}
+            eq_dict['RETURNS'] = {}
+            eq_dict['BODY'] = []
+
+            # NOTE: value generator
+            def val_generator(d):
+                '''Generate value'''
+                # check 3 |
+                if "|" in d:
+                    # split
+                    match_res = d.split("|")
+                    # check
+                    if len(match_res) != 3:
+                        raise ValueError(
+                            f"{d} must be 3 elements!")
+
+                    # key
+                    key = match_res[0].strip()
+                    # name
+                    name = key
+                    # symbol
+                    symbol = match_res[1].strip()
+                    # unit
+                    unit = match_res[2].strip()
+
+                    # set
+                    val = {
+                        'name': name,
+                        'symbol': symbol,
+                        'unit': unit
+                    }
+                else:
+                    raise Exception(
+                        f"Invalid value format! {d} must be 3 elements!")
+
+                return key, val
+
+            # SECTION: parse equation data
+            # loop through eq_data
+            for eq in eq_data:
+                # check
+                if isinstance(eq, str):
+                    # parse equation body
+                    body_ = eq
+
+                    # NOTE: parse to extract args
+                    # regex
+                    regex = r"args\['([^']+)'\]"
+                    # find all matches
+                    matches = re.findall(regex, body_)
+                    # check
+                    if matches:
+                        # loop through matches
+                        for match in matches:
+                            # ! val generator
+                            key, val = val_generator(match)
+                            symbol = val['symbol']
+                            # check
+                            if match not in eq_dict['ARGS']:
+                                # set
+                                eq_dict['ARGS'][key] = val
+                                # set body
+                                body_ = body_.replace(
+                                    f"'{match}'", f"'{symbol}'")
+
+                    # NOTE: parse to extract parameters
+                    # regex
+                    regex = r"parms\['([^']+)'\]"
+                    # find all matches
+                    matches = re.findall(regex, body_)
+                    # check
+                    if matches:
+                        # loop through matches
+                        for match in matches:
+                            # ! val generator
+                            key, val = val_generator(match)
+                            symbol = val['symbol']
+                            # check
+                            if match not in eq_dict['PARMS']:
+                                # set
+                                eq_dict['PARMS'][key] = val
+                                # set body
+                                body_ = body_.replace(
+                                    f"'{match}'", f"'{symbol}'")
+
+                    # NOTE: parse to extract returns
+                    # regex
+                    regex = r"res\['([^']+)'\]"
+                    # find all matches
+                    matches = re.findall(regex, body_)
+                    # check
+                    if matches:
+                        # loop through matches
+                        for match in matches:
+                            # ! val generator
+                            key, val = val_generator(match)
+                            symbol = val['symbol']
+                            # check
+                            if match not in eq_dict['RETURNS']:
+                                # set
+                                eq_dict['RETURNS'][key] = val
+                                # set body
+                                body_ = body_.replace(f"res['{match}']", "res")
+
+                # updated body
+                # set body
+                eq_dict['BODY'].append(body_)
+            # return
+            return eq_dict
+        except Exception as e:
+            raise Exception(f"parse_equation error! {e}")
+
+    def __eq_formatter(self, eqs: List):
+        '''
+        Format equations
+        '''
+        try:
+            # NOTE: eqs
+            eqs_formatted = []
+
+            # NOTE: check eqs
+            for eq in eqs:
+                # check keys
+                if "ARGS" not in eq or "PARMS" not in eq or "RETURNS" not in eq:
+                    # parse eq to generate equation structure
+                    eq_ = self.equation_formatter(eq)
+                    eqs_formatted.append(eq_)
+                else:
+                    eqs_formatted.append(eq)
+
+            # res
+            return eqs_formatted
+        except Exception as e:
+            raise Exception(f"equation formatting error! {e}")
+
+    def equation_formatter(self, equation_src: Dict | str) -> Dict[str, Any]:
+        '''
+        Check equation format
+
+        Parameters
+        ----------
+        equation_src : str | dict
+            equation source (file path string or dict)
+        '''
+        try:
+            # SECTION: path check
+            if isinstance(equation_src, str):
+                # check file
+                if not os.path.isfile(equation_src):
+                    raise FileNotFoundError(f"File not found: {equation_src}")
+
+                # NOTE: load equation file (yml format)
+                with open(equation_src, 'r') as f:
+                    equations = yaml.load(f, Loader=yaml.FullLoader)
+            elif isinstance(equation_src, dict):
+                # NOTE: check if dict
+                equations = equation_src
+
+            # SECTION: check equation format
+            # NOTE: check
+            if 'BODY' not in equations.keys():
+                raise ValueError("equation key `BODY` not found!")
+
+            # NOTE: get equation body
+            body_ = equations['BODY']
+
+            # NOTE: parse
+            parse_res = self.__parse_equation(body_)
+
+            # NOTE: check other equations
+            body_integral = equations.get('BODY-INTEGRAL', None)
+            body_first_derivative = equations.get(
+                'BODY-FIRST-DERIVATIVE', None)
+            body_second_derivative = equations.get(
+                'BODY-SECOND-DERIVATIVE', None)
+
+            # check
+            if body_integral is not None:
+                # check
+                if body_integral != 'None' and body_integral != 'NONE':
+                    # parse
+                    parse_ = self.__parse_equation(body_integral)
+                    # add to res
+                    parse_res['BODY-INTEGRAL'] = parse_['BODY']
+            else:
+                parse_res['BODY-INTEGRAL'] = None
+
+            if body_first_derivative is not None:
+                # check
+                if body_first_derivative != 'None' and body_first_derivative != 'NONE':
+                    # parse
+                    parse_ = self.__parse_equation(body_first_derivative)
+                    # add to res
+                    parse_res['BODY-FIRST-DERIVATIVE'] = parse_['BODY']
+            else:
+                parse_res['BODY-FIRST-DERIVATIVE'] = None
+
+            if body_second_derivative is not None:
+                # check
+                if body_second_derivative != 'None' and body_second_derivative != 'NONE':
+                    # parse
+                    parse_ = self.__parse_equation(body_second_derivative)
+                    # add to res
+                    parse_res['BODY-SECOND-DERIVATIVE'] = parse_['BODY']
+            else:
+                parse_res['BODY-SECOND-DERIVATIVE'] = None
+
+            # res
+            return parse_res
+        except FileNotFoundError:
+            raise FileNotFoundError(f"File not found: {equation_src}")
+        except Exception as e:
+            raise Exception(f"equation format checker error! {e}")
